# -*- coding: utf-8 -*-
import copy
from django.core.exceptions import ValidationError
from rest_framework import serializers, exceptions

from django.contrib.auth import models
from rest_framework.utils import model_meta


def clean_instance(attrs, instance, info):
    many_to_many = {}
    # TODO: verificare se copy è necessario
    attr_copy = copy.copy(attrs)
    for attr, value in attr_copy.items():
        if info.relations.get(attr) and info.relations.get(attr).to_many:
            many_to_many[attr] = attrs.pop(attr)
        else:
            setattr(instance, attr, value)
    try:
        instance.clean()
        attrs.update(many_to_many)
    except ValidationError as ve:
        raise exceptions.ValidationError(detail=ve.message_dict)
    return attrs


class PermissionSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Permission
        fields = '__all__'


class ThuxListSerializerMixin:
    pass


class ThuxRetrieveSerializerMixin:
    pass


class ThuxCreateSerializerMixin:

    def validate(self, attrs):
        super().validate(attrs)
        _attrs = copy.copy(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        many_to_many = {}
        for attr, value in _attrs.items():
            if info.relations.get(attr) and info.relations.get(attr).to_many:
                many_to_many[attr] = attrs.pop(attr)

        attrs["creator"] = self.context["request"].user
        attrs["last_modifier"] = self.context["request"].user
        instance = self.Meta.model(**attrs)
        try:
            instance.clean()
            attrs.update(many_to_many)
        except ValidationError as ve:
            raise exceptions.ValidationError(detail=ve.message_dict)
        return attrs


class ThuxUpdateSerializerMixin:

    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        return clean_instance(attrs, self.instance, info)


class ThuxPartialUpdateSerializerMixin(ThuxUpdateSerializerMixin):
    pass


class ThuxStatusSerializerMixin:

    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        status = attrs.get("status")
        attrs.clear()
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = status
        return clean_instance(attrs, self.instance, info)


class ThuxEnableSerializerMixin:

    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = 1
        return clean_instance(attrs, self.instance, info)


class ThuxDisableSerializerMixin:

    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = 0
        return clean_instance(attrs, self.instance, info)
